using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.Reporting
{
    internal class SliderFilterEditor : UITypeEditor, IFilter
    {
        #region Fields

        private readonly int _maxValue = 100;
        private readonly int _minValue;
        private Column _column;
        private int _currentValue;

        #endregion

        #region Constructors

        public SliderFilterEditor()
        {
        }

        public SliderFilterEditor(int minValue, int maxValue)
        {
            _minValue = minValue;
            _maxValue = maxValue;
        }

        #endregion

        #region IFilter Members

        public bool IsFiltered(Row row)
        {
            //Filter implementation
            if (_column != null)
            {
                //Get a value in the cell and compare with the current filter
                Cell cell = row[_column.Id];
                object value = cell != null ? cell.Value : null;
                if (value != null && value is int)
                {
                    //Return the result. True - filtered, false - visible
                    return !((int) value >= _currentValue);
                }
            }
            return false;
        }


        public event EventHandler<EventArgs> FilterUpdated;

        #endregion

        #region UITypeEditor implementation

        public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            //The value is a column object. Keep it!
            if (value is Column)
            {
                _column = (Column) value;
            }

            //If the editor can return three values:
            //1. the same object - the filter is unchanged. 
            //2. null - filter is removed 
            //3. IFilter object - the filter is set.
            //By default we return the same object
            object result = value;
            IWindowsFormsEditorService service = provider.GetService(typeof (IWindowsFormsEditorService)) as IWindowsFormsEditorService;
            if (service != null)
            {
                //Create some control. In our case - TrackBar that will be owned by the dropdown control
                using (TrackBar control = new TrackBar())
                {
                    //Set its width and style
                    control.Width = _column.Width;
                    control.TickStyle = TickStyle.TopLeft;
                    control.TickFrequency = Math.Max(1, (_maxValue - _minValue)/10);

                    //Set values
                    control.SetRange(_minValue, _maxValue);
                    control.Value = _currentValue;

                    //Subscribe for ValueChanged event
                    control.ValueChanged += delegate
                    {
                        //Update filter each time the value is changed, but do not close the control
                        _currentValue = control.Value;
                        if (FilterUpdated != null)
                        {
                            FilterUpdated(this, EventArgs.Empty);
                        }
                    };

                    //Subscribe for MouseUp event
                    control.MouseUp += delegate
                    {
                        //Close control when the user releases the button. A reference to IFilter object should be returned
                        result = this;
                        service.CloseDropDown();
                    };

                    //Start the editing process
                    service.DropDownControl(control);
                }
            }
            return result;
        }

        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.DropDown;
        }

        #endregion
    }
}