using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.CurrencyViewer
{
    /// <summary>
    /// Implementation of IDataAccessor that provides Grid an access to the CurrencyRate object
    /// </summary>
    public class CurrencyRow : IDataAccessor
    {
        #region Static constructor

        private static readonly Dictionary<string, CurrencyType> _currencyDico = new Dictionary<string, CurrencyType>();
        private readonly Array _currencyList = Enum.GetValues(typeof (CurrencyType));

        static CurrencyRow()
        {
            foreach (CurrencyType currency in Enum.GetValues(typeof (CurrencyType)))
            {
                _currencyDico.Add(currency.ToString(), currency);
            }
        }

        #endregion

        #region Fields

        private readonly CurrencyRate _rate;

        #endregion

        #region Constructor

        public CurrencyRow(CurrencyRate rate)
        {
            _rate = rate;
            rate.RateChanged += delegate(object sender, RateChangedEventArgs e)
            {
                if (FieldChanged != null)
                {
                    FieldChanged(this, new FieldChangedEventArgs(this, new CurrencyRowField(this, e.Currency)));
                }
            };
        }

        #endregion

        #region IDataAccessor Members

        public IEnumerator<IDataField> GetEnumerator()
        {
            return new Enumerator(this);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public event EventHandler<FieldChangedEventArgs> FieldChanged;

        IDataField IDataAccessor.this[string id]
        {
            get
            {
                CurrencyType currency;
                if (!string.IsNullOrEmpty(id) && _currencyDico.TryGetValue(id, out currency))
                {
                    return new CurrencyRowField(this, currency);
                }
                return null;
            }
        }

        IDataField IDataAccessor.this[int index]
        {
            get
            {
                if (index >= 0 && index < _currencyList.Length)
                {
                    return new CurrencyRowField(this, (CurrencyType) _currencyList.GetValue(index));
                }
                return null;
            }
        }

        public bool SupportsIdentifiers
        {
            get { return true; }
        }

        public bool SupportsNotifications
        {
            get { return true; }
        }

        public int Count
        {
            get { return _currencyList.Length; }
        }

        public object DataObject
        {
            get { return _rate; }
        }

        public IList<IDataField> HierarchicalFields
        {
            get { return null; }
        }

        #endregion

        #region Nested type: CurrencyRowField

        private class CurrencyRowField : IDataField
        {
            private static readonly IFormat _format = new DoubleFormat(2, false, false);
            private readonly CurrencyType _currency;
            private readonly CurrencyRow _owner;

            public CurrencyRowField(CurrencyRow owner, CurrencyType currency)
            {
                _owner = owner;
                _currency = currency;
            }

            #region IDataField Members

            public void NotifyUpdate()
            {
                if (_owner.FieldChanged != null)
                {
                    _owner.FieldChanged(_owner, new FieldChangedEventArgs(_owner, this));
                }
            }

            public string Id
            {
                get { return _currency.ToString(); }
            }

            public int Index
            {
                get { return (int) _currency; }
            }

            public string DisplayName
            {
                get { return string.Format("{0}/{1}", _owner._rate.MainCurrency, _currency); }
            }

            public string Description
            {
                get { return null; }
            }

            public IFormat Format
            {
                get { return _format; }
            }

            public bool Browsable
            {
                get { return true; }
            }

            public bool Bindable
            {
                get { return true; }
            }

            public bool Readonly
            {
                get
                {
                    if (Equals(_owner._rate.MainCurrency, _currency))
                    {
                        return true;
                    }
                    return false;
                }
            }

            public UITypeEditor Editor
            {
                get { return null; }
            }

            public Type FieldType
            {
                get { return typeof (double); }
            }

            public IDataAccessor DataAccessor
            {
                get { return _owner; }
            }

            public object Value
            {
                get { return _owner._rate[_currency].Rate; }
                set { _owner._rate[_currency].Rate = (double) value; }
            }

            public bool IsComposite
            {
                get { return false; }
            }

            public bool IsHierarchical
            {
                get { return false; }
            }

            #endregion
        }

        #endregion

        #region Nested type: Enumerator

        private class Enumerator : IEnumerator<IDataField>
        {
            #region Fields

            private readonly IEnumerator _baseEnumerator;
            private readonly CurrencyRow _owner;

            #endregion

            public Enumerator(CurrencyRow owner)
            {
                _baseEnumerator = Enum.GetValues(typeof (CurrencyType)).GetEnumerator();
                _owner = owner;
            }

            #region IEnumerator<IDataField> Members

            public void Dispose()
            {
            }

            public bool MoveNext()
            {
                return _baseEnumerator.MoveNext();
            }

            public void Reset()
            {
                _baseEnumerator.Reset();
            }

            public IDataField Current
            {
                get { return new CurrencyRowField(_owner, (CurrencyType) _baseEnumerator.Current); }
            }

            object IEnumerator.Current
            {
                get { return Current; }
            }

            #endregion
        }

        #endregion
    }
}
