using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;
using Appearance = Dapfor.Net.Ui.Appearance;
using StringFormat = Dapfor.Net.Formats.StringFormat;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.CurrencyViewer
{
    [Example("Currency Viewer", "Miscellaneous")]
    public partial class CurrencyForm : UserControl
    {
        private readonly Appearance _bottomLeft = new Appearance(Color.Black, Color.AliceBlue, true, Color.FromArgb(214, 234, 254), GradientDirection.Vertical);
        private readonly Color _hotCellColor = Color.FromArgb(70, Color.Blue);
        private readonly Color _hotMirroredCellColor = Color.FromArgb(120, Color.Orange);
        private readonly Color _hotMirroredRowColumnColor = Color.FromArgb(40, Color.Orange);
        private readonly Color _hotMirroredSelectorColor = Color.FromArgb(120, Color.Orange);
        private readonly Color _hotRowColumnColor = Color.FromArgb(40, Color.Blue);
        private readonly Color _hotSelectorColor = Color.FromArgb(120, Color.Blue);
        private readonly Random _r = new Random();
        private readonly Dictionary<CurrencyType, CurrencyRate> _rates = new Dictionary<CurrencyType, CurrencyRate>();
        private readonly Timer _timer = new Timer();
        private readonly Appearance _topRight = new Appearance(Color.Black, Color.FromArgb(255, 253, 242), true, Color.FromArgb(255, 249, 209), GradientDirection.Vertical);

        public CurrencyForm()
        {
            InitializeComponent();

            //Prepare a table of currencies with its rates
            CreateData();

            //Populate header with columns
            Header header = grid.Headers[0];
            foreach (CurrencyType currency in Enum.GetValues(typeof (CurrencyType)))
            {
                Column column = new Column(currency.ToString(), currency.ToString(), 50, ContentAlignment.MiddleCenter, true, true);
                column.Format = new StringFormat("0.###");
                column.Sortable = false;
                column.Editable = false;
                header.Add(column);
            }
            
            PopulateGrid();

            grid.Headers[0].AdjustColumns(true, false);

            _timer.Tick += delegate { UpdateRate(); };
            _timer.Interval = 1;
            _timer.Start();
        }


        #region Data updating routines

        public void UpdateRate()
        {
            CurrencyType currency1 = (CurrencyType)Enum.ToObject(typeof(CurrencyType), _r.Next(_rates.Count));
            CurrencyType currency2 = (CurrencyType)Enum.ToObject(typeof(CurrencyType), _r.Next(_rates.Count));
            if (!Equals(currency1, currency2))
            {
                double rate = _rates[currency1][currency2].Rate;
                double coeff = 1 / ((double)(_r.Next(200, 400)));
                coeff = _r.Next() % 2 == 0 ? coeff : -coeff;
                rate *= (1 + coeff);
                rate = rate == 0 ? _rates[currency1][currency2].Rate : rate;
                _rates[currency1][currency2].Rate = rate;
                _rates[currency2][currency1].Rate = 1 / rate;
            }
        }

        #endregion

        #region Data creation routines

        private void CreateData()
        {
            foreach (CurrencyType currency in Enum.GetValues(typeof (CurrencyType)))
            {
                _rates.Add(currency, new CurrencyRate(currency));
            }

            _rates[CurrencyType.USD][CurrencyType.USD].Rate = 1;
            _rates[CurrencyType.USD][CurrencyType.CAD].Rate = 1/0.96367;
            _rates[CurrencyType.USD][CurrencyType.EUR].Rate = 1/1.2852;
            _rates[CurrencyType.USD][CurrencyType.GBP].Rate = 1/1.5269;
            _rates[CurrencyType.USD][CurrencyType.JPY].Rate = 1/0.011496;
            _rates[CurrencyType.USD][CurrencyType.AUD].Rate = 1/0.8838;
            _rates[CurrencyType.USD][CurrencyType.ZAR].Rate = 1/0.132371;
            _rates[CurrencyType.USD][CurrencyType.SEK].Rate = 1/0.135685;
            _rates[CurrencyType.USD][CurrencyType.CHF].Rate = 1/0.951022;
            _rates[CurrencyType.USD][CurrencyType.DKK].Rate = 1/0.172467;
            _rates[CurrencyType.USD][CurrencyType.HKD].Rate = 1/0.128602;
            _rates[CurrencyType.USD][CurrencyType.INR].Rate = 1/0.021157;
            _rates[CurrencyType.USD][CurrencyType.ILS].Rate = 1/0.259168;
            _rates[CurrencyType.USD][CurrencyType.ISK].Rate = 1/0.006138;
            _rates[CurrencyType.USD][CurrencyType.RUB].Rate = 1/0.032861;
            _rates[CurrencyType.USD][CurrencyType.MAD].Rate = 1/0.115765;


            //Populate cross rates
            foreach (KeyValuePair<CurrencyType, CurrencyRate> rate in _rates)
            {
                CurrencyType currency1 = rate.Key;
                if (currency1 != CurrencyType.USD)
                {
                    foreach (CurrencyType currency2 in Enum.GetValues(typeof(CurrencyType)))
                    {
                        double usdRate1 = _rates[CurrencyType.USD][currency1].Rate;
                        double usdRate2 = _rates[CurrencyType.USD][currency2].Rate;
                        if (usdRate1 > 0 && usdRate2 > 0)
                        {
                            rate.Value[currency2].Rate = usdRate2 / usdRate1;
                        }
                    }
                }
            }
        }

        private void PopulateGrid()
        {
            foreach (KeyValuePair<CurrencyType, CurrencyRate> rate in _rates)
            {
                grid.Rows.Add(new CurrencyRow(rate.Value));
            }
        }

        #endregion

        #region Private methods

        private Cell GetMirroredCell(Cell cell)
        {
            if (cbHighlightCell.Checked && cbMirroir.Checked && cell != null && cell.Row != null && cell.Column != null && cell.Row.DataObject != null)
            {
                CurrencyType c1 = ((CurrencyRate) cell.Row.DataObject).MainCurrency;
                CurrencyType c2 = (CurrencyType) Enum.Parse(typeof (CurrencyType), cell.Column.Id);
                Row row = grid.Rows[(int) c2];
                return row != null ? row[c1.ToString()] : null;
            }
            return null;
        }

        private static void InvalidateCell(Cell cell)
        {
            if (cell != null)
            {
                cell.Invalidate();
                if (cell.Row != null)
                {
                    cell.Row.Invalidate();
                }
                if (cell.Column != null)
                {
                    cell.Column.InvalidateCells();
                    cell.Column.Invalidate();
                }
            }
        }

        #endregion

        #region Private event handlers

        private void OnCellLeave(object sender, GridCellEventArgs e)
        {
            InvalidateCell(e.Cell);
            InvalidateCell(GetMirroredCell(e.Cell));
        }

        private void OnCellEnter(object sender, GridCellEventArgs e)
        {
            InvalidateCell(e.Cell);
            InvalidateCell(GetMirroredCell(e.Cell));
        }

        private void OnPaintColumnCaption(object sender, PaintColumnCaptionEventArgs e)
        {
            //Hot column
            if (cbHighlightCell.Checked)
            {
                Point pt = e.Grid.PointToClient(Cursor.Position);
                if (pt.X >= e.VisibleBounds.Left && pt.X < e.VisibleBounds.Right && e.Grid.HitTests.CellTest(pt) != null)
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotSelectorColor);
                    e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotSelectorColor);
                }
                else
                {
                    //Mirrored column
                    Cell cell = GetMirroredCell(e.Grid.HitTests.CellTest(pt));
                    if (cell != null && cell.Column != null && Equals(cell.Column, e.Column))
                    {
                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotMirroredSelectorColor);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotMirroredSelectorColor);
                    }
                }
            }
        }

        private void OnPaintRow(object sender, PaintRowEventArgs e)
        {
            if (cbHighlightCell.Checked)
            {
                //Hot selector
                Point pt = e.Grid.PointToClient(Cursor.Position);
                if (e.Row.Bounds.Contains(pt) && (e.Grid.HitTests.CellTest(pt) != null))
                {
                    e.RowSelectorAppearance.BackColor = PaintHelper.AlphaBlend(e.RowSelectorAppearance.BackColor, _hotSelectorColor);
                    e.RowSelectorAppearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.RowSelectorAppearance.GradientEndBackColor, _hotSelectorColor);
                }
                else
                {
                    //Mirrored cell
                    Cell cell = GetMirroredCell(e.Grid.HitTests.CellTest(pt));
                    if (cell != null && cell.Row != null && cell.Row.VisibleIndex == e.Row.VisibleIndex)
                    {
                        e.RowSelectorAppearance.BackColor = PaintHelper.AlphaBlend(e.RowSelectorAppearance.BackColor, _hotMirroredSelectorColor);
                        e.RowSelectorAppearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.RowSelectorAppearance.GradientEndBackColor, _hotMirroredSelectorColor);
                    }
                }
            }


            e.Parts &= e.Parts ^ PaintPart.RowSelectorImages;
            e.PaintAll();
            e.Handled = true;

            //Draw text over row selector
            if (e.Row != null)
            {
                using (System.Drawing.StringFormat sf = new System.Drawing.StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    e.Graphics.DrawString(((CurrencyRate)e.Row.DataObject).MainCurrency.ToString(), grid.Headers[0].Appearance.ColumnCaptionFont, Brushes.Black, e.RowSelectorBounds, sf);
                }
            }
        }

        private void OnRowUpdated(object sender, GridRowUpdateEventArgs e)
        {
            CurrencyType id = (CurrencyType)Enum.Parse(typeof(CurrencyType), e.DataField.Id);
            CurrencyRate rate = (CurrencyRate)e.Row.DataObject;
            Color color = rate[id].Rate > rate[id].PrevRate ? Color.Green : Color.Red;
            e.Row[e.DataField.Id].Highlight(TimeSpan.FromMilliseconds(1000), Color.FromArgb(30, color));
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Row != null && e.Cell.Column != null)
            {
                CurrencyRate rate = (CurrencyRate)e.Cell.Row.DataObject;
                CurrencyType currency1 = (CurrencyType)Enum.Parse(typeof(CurrencyType), e.Cell.Column.Id);
                if (!Equals(currency1, rate.MainCurrency) && cbImages.Checked)
                {
                    e.Image = rate[currency1].Rate > rate[currency1].PrevRate ? Resources.arrow_up_green : Resources.arrow_down_red;
                    e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;
                }

                if ((int)currency1 > e.Cell.Row.VisibleIndex)
                {
                    e.Appearance.ForeColor = _topRight.ForeColor;
                    e.Appearance.BackColor = _topRight.BackColor;
                    e.Appearance.GradientEndBackColor = _topRight.GradientEndBackColor;
                    e.Appearance.GradientDirection = _topRight.GradientDirection;
                    e.Appearance.GradientEnabled = _topRight.GradientEnabled;
                }
                else if ((int)currency1 < e.Cell.Row.VisibleIndex)
                {
                    e.Appearance.ForeColor = _bottomLeft.ForeColor;
                    e.Appearance.BackColor = _bottomLeft.BackColor;
                    e.Appearance.GradientEndBackColor = _bottomLeft.GradientEndBackColor;
                    e.Appearance.GradientDirection = _bottomLeft.GradientDirection;
                    e.Appearance.GradientEnabled = _bottomLeft.GradientEnabled;
                }
                else
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, Color.FromArgb(40, Color.Blue));
                    e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, Color.FromArgb(40, Color.Blue));
                }

                if (cbHighlightCell.Checked)
                {
                    //Hot column and row
                    Point pt = e.Grid.PointToClient(Cursor.Position);
                    if (e.Cell.Row.Bounds.Contains(pt))
                    {
                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotRowColumnColor);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotRowColumnColor);
                    }
                    if (pt.X > e.Cell.VisibleBounds.Left && pt.X < e.Cell.VisibleBounds.Right && e.Grid.ClientRectangle.Contains(pt) && e.Grid.HitTests.CellTest(pt) != null)
                    {
                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotRowColumnColor);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotRowColumnColor);
                    }

                    //Mirorred column and row
                    Cell mirorredCell = GetMirroredCell(e.Grid.HitTests.CellTest(pt));
                    if (mirorredCell != null && mirorredCell.Row.VisibleIndex == e.Cell.Row.VisibleIndex)
                    {
                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotMirroredRowColumnColor);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotMirroredRowColumnColor);
                    }
                    if (mirorredCell != null && mirorredCell.Column.VisibleIndex == e.Cell.Column.VisibleIndex)
                    {
                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, _hotMirroredRowColumnColor);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, _hotMirroredRowColumnColor);
                    }


                    //hot cells and a frame around them
                    Cell hotCell = e.Grid.HitTests.CellTest(e.Grid.PointToClient(Cursor.Position));
                    if (Equals(e.Cell, hotCell) || Equals(e.Cell, mirorredCell))
                    {
                        Color color = Equals(e.Cell, mirorredCell) ? _hotMirroredCellColor : _hotCellColor;

                        e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, color);
                        e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, color);
                        e.PaintAll();
                        e.Handled = true;
                        e.Render.DrawFrame(Color.Black, e.Cell.VirtualBounds, ButtonBorderStyle.Solid, e.Graphics);
                    }
                }
            }
        }

        private void OnImagesCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].AdjustColumns(true, false);
            grid.Invalidate(false);
        }

        private void OnRefreshCheckedChanged(object sender, EventArgs e)
        {
            _timer.Enabled = cbRefresh.Checked;
        }

        private void OnHighlightingCheckedChanged(object sender, EventArgs e)
        {
            grid.Highlighting.Enabled = cbHighlighting.Checked;
        }

        #endregion

    }
}