using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Performance.FilterPerformance
{
    [Example("Filter performance", "Performance")]
    public partial class FilterPerformanceDemoForm : UserControl
    {
        #region Fields

        private readonly Filter _filter;
        private readonly Random _random = new Random();

        private int? _matchValue;
        private string _matchString;
        private string _regExpression;

        #endregion

        public FilterPerformanceDemoForm()
        {
            //Initialize the grid
            InitializeComponent();

            //Set filter
            _filter = new Filter(IsFiltered);
            grid.Filter = _filter;

            //Set initial parameters
            tbFilter.Text = "12";
            tbRowCount.Text = "1000";
            rbValuesComaping.Checked = true;
        }

        #region Private mathods

        private bool IsFiltered(Row row)
        {
            bool filtered = false;

            //The grid calls this callback in the GUI thread each time when the value of the data object in changed
            foreach (Column column in grid.Headers[0].VisibleColumns)
            {
                //Check whether the cell matches to the filter
                filtered = !IsMatched(row[column.Id]);

                //The row can be shown - do not verify remaining columns
                if (!filtered)
                {
                    break;
                }
            }

            //The row is not filtered
            return filtered;
        }

        private bool IsMatched(Cell cell)
        {
            bool matched = true;
            if (cell != null)
            {
                //Match values
                if (rbValuesComaping.Checked && _matchValue != null)
                {
                    matched = Equals(_matchValue.Value, cell.Value);    
                }
                    //Match strings
                else if (rbStringsComparing.Checked && !string.IsNullOrEmpty(_matchString))
                {
                    matched = cell.Text.Contains(_matchString);
                }
                    //Match to regular expression
                else if (rbRegularExpresisons.Checked && _regExpression != null)
                {
                    try
                    {
                        matched = Regex.IsMatch(cell.Text, "^" + _regExpression, RegexOptions.Singleline | RegexOptions.IgnoreCase);
                    }
                    catch
                    {
                        matched = true;
                    }
                }
            }
            return matched;
        }

        #endregion

        #region Private event handlers

        private void OnGridPaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Row != null && e.Cell.Column != null)
            {
                //Highlight the cell if it matches to the filter
                if (IsMatched(e.Cell) &&
                    ((rbValuesComaping.Checked && _matchValue != null) ||
                     (rbStringsComparing.Checked && !string.IsNullOrEmpty(_matchString)) ||
                     (rbRegularExpresisons.Checked && !string.IsNullOrEmpty(_regExpression))))
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, Color.FromArgb(100, Color.Red));
                    e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, Color.FromArgb(100, Color.Red));
                }
            }
        }

        private void OnFilterUpdated(object sender, EventArgs e)
        {
            //Get a value to match rows
            _matchValue = null;
            int v;
            if (!string.IsNullOrEmpty(tbFilter.Text) &&
                int.TryParse(tbFilter.Text.Trim(), NumberStyles.Number | NumberStyles.AllowThousands | NumberStyles.AllowLeadingWhite, CultureInfo.CurrentCulture, out v))
            {
                _matchValue = v;
            }
            
            //String comparing
            _matchString = tbFilter.Text;

            //Regular expression
            _regExpression = _matchString ?? string.Empty;
            _regExpression = _regExpression.Replace("*", ".*");
            _regExpression = _regExpression.Replace("?", ".");

            //Measure the time to do the filter job
            DateTime dt = DateTime.Now;
            
            //Notify the Grid about the filter contitions changings
            grid.FilterRefresh();
            TimeSpan ts = DateTime.Now - dt;

            lbVisibleRows.Text = grid.Rows.Count.ToString();
            lbJob.Text = ts.TotalMilliseconds.ToString();
            grid.Invalidate(false);
        }

        private void OnRowCountChanged(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRowCount.Text) && int.TryParse(tbRowCount.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                //Modify the count of dataobjects 
                if (grid.Nodes.Count < count)
                {
                    //Add new items to the Grid and dataObjects container
                    long itemsToAdd = count - grid.Nodes.Count;
                    for (int i = 0; i < itemsToAdd; ++i)
                    {
                        DataObject dataObject = new DataObject(_random);
                        grid.Rows.Add(dataObject);
                    }
                }
                else if (count < grid.Nodes.Count)
                {
                    //Create a list of items to be removed
                    int toRemove = grid.Nodes.Count - count;
                    List<Row> rowsToRemove = new List<Row>();
                    for (int i = 0; i < toRemove; ++i)
                    {
                        rowsToRemove.Add(grid.Nodes[grid.Nodes.Count - (1 + i)]);
                    }
                    foreach (Row row in rowsToRemove)
                    {
                        grid.Rows.Remove(row);
                    }
                }
            }
            else if (string.IsNullOrEmpty(tbRowCount.Text))
            {
                grid.Rows.Clear();
            }
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        #endregion
    }
}