using System;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Performance.RealtimeSort
{
    [Example("Real-time sorting and filtering", "Performance")]
    public partial class RealtimeSortDemoForm : UserControl
    {
        #region Fields

        private readonly Random _random = new Random();

        private Timer _guiTimer;
        private DateTime _lastTimeStamp = DateTime.MinValue;
        private int _updatesPerSecond;
        private int _filterMinVal;
        private int _filterMaxVal;

        #endregion

        public RealtimeSortDemoForm()
        {
            InitializeComponent();

            lbGridSize.Text = string.Empty;

            //Start Gui timer to update data objects
            _guiTimer = new Timer();
            _guiTimer.Interval = 10;
            _guiTimer.Tick += OnGuiTimerTick;

            
            //Set a row count and updating frequency
            tbRowCount.Text = "2000";
            tbUpdatePerSec.Text = "500";

            //Set some sort in the grid
            grid.Headers[0].VisibleColumns[2].SortDirection = SortDirection.Ascending;
            grid.Headers[0].VisibleColumns[4].SortDirection = SortDirection.Descending;
            grid.Headers[0].VisibleColumns[5].SortDirection = SortDirection.Descending;

            //Set a filter to the grid
            grid.Filter = new Filter(IsFiltered);
            cbMinValue.Checked = true;
            cbMaxValue.Checked = true;
            tbMinValue.Text = "100";
            tbMaxValue.Text = "800";
        }

        #region Private event handlers

        private void OnFilterChanged(object sender, EventArgs e)
        {
            UpdateFilter();
        }

        private void UpdateDataObject(DataObject dataObject)
        {
            int newValue = _random.Next()%2000;
            for (int i = 0; i < 5; ++i)
            {
                newValue = Math.Min(newValue, _random.Next()%2000);
            }

            switch (_random.Next() % 10)
            {
                case 0:
                    dataObject.Value0 = newValue;
                    break;
                case 1:
                    dataObject.Value1 = newValue;
                    break;
                case 2:
                    dataObject.Value2 = newValue;
                    break;
                case 3:
                    dataObject.Value3 = newValue;
                    break;
                case 4:
                    dataObject.Value4 = newValue;
                    break;
                case 5:
                    dataObject.Value5 = newValue;
                    break;
                case 6:
                    dataObject.Value6 = newValue;
                    break;
                case 7:
                    dataObject.Value7 = newValue;
                    break;
                case 8:
                    dataObject.Value8 = newValue;
                    break;
                case 9:
                    dataObject.Value9 = newValue;
                    break;
            }
        }

        private void OnRowCountChanged(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRowCount.Text) && int.TryParse(tbRowCount.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                //Modify count of dataobjects in the dataObjects container and in the grid
                if (grid.Nodes.Count < count)
                {
                    //Add new items to the Grid and dataObjects container
                    long itemsToAdd = count - grid.Nodes.Count;
                    for (int i = 0; i < itemsToAdd; ++i)
                    {
                        DataObject dataObject = new DataObject(_random);
                        grid.Rows.Add(dataObject);
                    }
                }
                else if (count < grid.Nodes.Count)
                {
                    //Create a list of items to be removed
                    int toRemove = grid.Nodes.Count - count;

                    for (int i = 0; i < toRemove; ++i)
                    {
                        grid.Rows.Remove(grid.Nodes[grid.Nodes.Count - 1]);
                    }
                }
            }
            else if (string.IsNullOrEmpty(tbRowCount.Text))
            {
                grid.Rows.Clear();
            }

            SetUpdatesCountInVisibleBounds();
        }

        private void OnGuiTimerTick(object sender, EventArgs e)
        {
            DateTime current = DateTime.UtcNow;
            TimeSpan ts = current - _lastTimeStamp;

            if (_updatesPerSecond > 0 && grid.Rows.Count > 0)
            {
                int updatesCount = (int) ((ts.TotalMilliseconds*_updatesPerSecond)/1000);
                updatesCount = Math.Max(1, updatesCount);

                for (int i = 0; i < updatesCount; ++i)
                {
                    Row row = grid.Nodes[_random.Next() % grid.Nodes.Count];
                    UpdateDataObject((DataObject) row.DataObject);
                }
            }

            _lastTimeStamp = DateTime.UtcNow;
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        private void OnUpdatesCountChanged(object sender, EventArgs e)
        {
            //Set a new amount of updates per second
            if (!string.IsNullOrEmpty(tbUpdatePerSec.Text))
            {
                int count;
                if (int.TryParse(tbUpdatePerSec.Text, out count))
                {
                    _updatesPerSecond = count;
                }
            }
            else
            {
                _updatesPerSecond = 0;
            }
            InitializeTimers();
            SetUpdatesCountInVisibleBounds();
        }

        private void OnGridSizeChanged(object sender, EventArgs e)
        {
            lbGridSize.Text = string.Format("{0} x {1}", grid.Size.Width, grid.Size.Height);
            SetUpdatesCountInVisibleBounds();
        }

        #endregion

        #region Private methods

        private bool IsFiltered(Row row)
        {
            //The grid calls this callback in the GUI thread each time when the value of the data object in changed
            foreach (Column column in grid.Headers[0].VisibleColumns)
            {
                int value = (int)row[column.Id].Value;
                //Check for the min bounds
                if (tbMinValue.Enabled)
                {
                    if (value < _filterMinVal) return true;
                }
                //Check for the max bounds
                if (tbMaxValue.Enabled)
                {
                    if (value > _filterMaxVal) return true;
                }
            }

            //The row is not filtered
            return false;
        }

        private void UpdateFilter()
        {
            tbMinValue.Enabled = cbMinValue.Checked;
            tbMaxValue.Enabled = cbMaxValue.Checked;

            int.TryParse(tbMinValue.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out _filterMinVal);
            int.TryParse(tbMaxValue.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out _filterMaxVal);
            grid.FilterRefresh();
        }

        private void InitializeTimers()
        {
            int interval = _updatesPerSecond > 0 ? 1000/_updatesPerSecond : 10;
            interval = Math.Min(1000, interval);
            interval = Math.Max(10, interval);


            _lastTimeStamp = DateTime.UtcNow;

            //Start a GUI timer
            _guiTimer.Interval = interval;
            _guiTimer.Start();
        }

        private void SetUpdatesCountInVisibleBounds()
        {
            int displayedRows = grid.Rows.TopVisibleRow != null && grid.Rows.BottomVisibleRow != null
                                    ? grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex + 1
                                    : 0;
            int visibleUpdates = grid.Rows.Count != 0 ? _updatesPerSecond*displayedRows/grid.Rows.Count : 0;
            lbVisibleRows.Text = string.Format("{0} upd/sec", visibleUpdates);
        }

        #endregion
    }
}