using System;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Performance.RealtimeUpdates
{
    [Example("Real-time updates", "Performance")]
    public partial class RealtimeUpdatesDemoForm : UserControl
    {
        #region Delegates

        public delegate void CallBack();

        #endregion

        #region Fields

        private readonly object _synchro = new object();
        private Timer _guiTimer;
        private Timer _guiAvgTimer;
        private DateTime _lastTimeStamp = DateTime.MinValue;
        private System.Threading.Timer _nonGuiTimer;
        private Random _random = new Random();
        private volatile int _updatesPerSecond;

        #endregion

        public RealtimeUpdatesDemoForm()
        {
            InitializeComponent();

            lbGridSize.Text = string.Empty;
            lbAvgTime.Text = string.Empty;
            lbAvgCount.Text = string.Empty;

            //Start Gui timer
            _guiTimer = new Timer();
            _guiTimer.Interval = 10;
            _guiTimer.Tick += OnGuiTimerTick;

            //Display average time, passed in the OnPaint method each second
            _guiAvgTimer = new Timer();
            _guiAvgTimer.Interval = 1000;
            _guiAvgTimer.Tick += OnGuiAvgTimerTick;
            _guiAvgTimer.Start();


            tbRowCount.Text = "2000";
            tbUpdatePerSec.Text = "5000";
            rbGuiThread.Checked = true;
        }

        #region Private event handlers

        private void OnGuiAvgTimerTick(object sender, EventArgs e)
        {
            lbAvgTime.Text = string.Format("{0:0.00}", grid.AverageTime);
            lbAvgCount.Text = string.Format("{0}", grid.AverageCallsPerSecond);
            
        }

        private void OnRowCountChanged(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRowCount.Text) && int.TryParse(tbRowCount.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                //Modify count of dataobjects in the dataObjects container and in the grid
                lock (_synchro)
                {
                    if (grid.Nodes.Count < count)
                    {
                        //Add new items to the Grid and dataObjects container
                        long itemsToAdd = count - grid.Nodes.Count;
                        for (int i = 0; i < itemsToAdd; ++i)
                        {
                            DataObject dataObject = new DataObject(_random);
                            grid.Rows.Add(dataObject);
                        }
                    }
                    else if (count < grid.Nodes.Count)
                    {
                        //Create a list of items to be removed
                        int toRemove = grid.Nodes.Count - count;

                        for (int i = 0; i < toRemove; ++i)
                        {
                            grid.Rows.RemoveAt(grid.Rows.Count - 1);
                        }
                    }
                }
            }
            else if (string.IsNullOrEmpty(tbRowCount.Text))
            {
                lock (_synchro)
                {
                    grid.Rows.Clear();
                }
            }

            SetUpdatesCountInVisibleBounds();
        }

        private void OnSourceCheckedChanged(object sender, EventArgs e)
        {
            InitializeTimers();
        }

        private void OnNonGuiTimerTick(object state)
        {
            UpdateItems();
        }

        private void OnGuiTimerTick(object sender, EventArgs e)
        {
            UpdateItems();
        }

        private void OnUpdatesCountChanged(object sender, EventArgs e)
        {
            //Set a new amount of updates per second
            if (!string.IsNullOrEmpty(tbUpdatePerSec.Text))
            {
                int count;
                if (int.TryParse(tbUpdatePerSec.Text, out count))
                {
                    _updatesPerSecond = count;
                }
            }
            else
            {
                _updatesPerSecond = 0;
            }
            InitializeTimers();
            SetUpdatesCountInVisibleBounds();
        }

        private void OnGridSizeChanged(object sender, EventArgs e)
        {
            lbGridSize.Text = string.Format("{0} x {1}", grid.Size.Width, grid.Size.Height);
            SetUpdatesCountInVisibleBounds();
        }

        #endregion

        #region Private methods

        private void UpdateItems()
        {
            DateTime current = DateTime.UtcNow;
            TimeSpan ts = current - _lastTimeStamp;

            if (_updatesPerSecond > 0 && grid.Rows.Count > 0)
            {
                int updatesCount = (int)((ts.TotalMilliseconds * _updatesPerSecond) / 1000);
                updatesCount = Math.Max(1, updatesCount);

                for (int i = 0; i < updatesCount; ++i)
                {
                    Row row = grid.Rows[GetRandom(grid.Rows.Count)];
                    if (row != null)
                    {
                        UpdateDataObject((DataObject)row.DataObject);
                    }
                }
            }


            _lastTimeStamp = DateTime.UtcNow;
        }

        private void UpdateDataObject(DataObject dataObject)
        {
            switch (GetRandom(10))
            {
                case 0:
                    dataObject.Value0 = GetRandom(0x7FFFFFFF);
                    break;
                case 1:
                    dataObject.Value1 = GetRandom(0x7FFFFFFF);
                    break;
                case 2:
                    dataObject.Value2 = GetRandom(0x7FFFFFFF);
                    break;
                case 3:
                    dataObject.Value3 = GetRandom(0x7FFFFFFF);
                    break;
                case 4:
                    dataObject.Value4 = GetRandom(0x7FFFFFFF);
                    break;
                case 5:
                    dataObject.Value5 = GetRandom(0x7FFFFFFF);
                    break;
                case 6:
                    dataObject.Value6 = GetRandom(0x7FFFFFFF);
                    break;
                case 7:
                    dataObject.Value7 = GetRandom(0x7FFFFFFF);
                    break;
                case 8:
                    dataObject.Value8 = GetRandom(0x7FFFFFFF);
                    break;
                case 9:
                    dataObject.Value9 = GetRandom(0x7FFFFFFF);
                    break;
            }
        }

        private void InitializeTimers()
        {
            //Limit max number of updates from the non-GUI thread
            if (_updatesPerSecond > 5000 && rbNonGuiThread.Checked)
            {
                tbUpdatePerSec.Text = "5000";
            }

            int interval = _updatesPerSecond > 0 ? 1000 / _updatesPerSecond : 10;
            interval = Math.Min(1000, interval);
            interval = Math.Max(10, interval);


            _lastTimeStamp = DateTime.UtcNow;
            if (rbGuiThread.Checked)
            {
                //Stop a non-GUI timer
                if (_nonGuiTimer != null)
                {
                    _nonGuiTimer.Dispose();
                    _nonGuiTimer = null;
                }
                //Start a GUI timer
                _guiTimer.Interval = interval;
                _guiTimer.Start();
            }
            else if (rbNonGuiThread.Checked)
            {
                //Stop a GUI timer
                _guiTimer.Stop();

                if (_nonGuiTimer != null)
                {
                    _nonGuiTimer.Dispose();
                }

                //Start the a GUI timer
                _nonGuiTimer = new System.Threading.Timer(OnNonGuiTimerTick, null, 0, interval);
            }
        }

        private void SetUpdatesCountInVisibleBounds()
        {
            int displayedRows = grid.Rows.TopVisibleRow != null && grid.Rows.BottomVisibleRow != null
                                    ? grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex + 1
                                    : 0;
            int visibleUpdates = grid.Rows.Count != 0 ? _updatesPerSecond * displayedRows / grid.Rows.Count : 0;
            lbVisibleUpdates.Text = string.Format("{0} upd/sec", visibleUpdates);
        }

        private int GetRandom(int max)
        {
            int result = _random.Next();
            if (result == 0)
            {
                _random = new Random();
            }
            return max > 0 ? result%max : 0;
        }

        #endregion
    }
}